package org.jbundle.base.field;

/**
 * @(#)PasswordField.java 0.00 12-Feb-97 Don Corley
 *
 * Copyright (c) 2009 tourapp.com. All Rights Reserved.
 *      don@tourgeek.com
 */

import org.jbundle.base.db.Record;
import org.jbundle.base.screen.model.BasePanel;
import org.jbundle.base.screen.model.SPasswordField;
import org.jbundle.base.screen.model.ScreenField;
import org.jbundle.base.screen.model.TopScreen;
import org.jbundle.base.screen.model.util.ScreenLocation;
import org.jbundle.thin.base.db.Converter;


/**
 * PasswordField - BaseField definition for a currency field - Saved as text internally.
 *
 * @version 1.0.0
 * @author    Don Corley
 */
public class PasswordField extends StringField
{
	private static final long serialVersionUID = 1L;

    /**
     * Constructor.
     */
    public PasswordField()
    {
        super();
    }
    /**
     * Constructor.
     * @param record The parent record.
     * @param strName The field name.
     * @param iDataLength The maximum string length (pass -1 for default).
     * @param strDesc The string description (usually pass null, to use the resource file desc).
     * @param strDefault The default value (if object, this value is the default value, if string, the string is the default).
     */
    public PasswordField(Record record, String strName, int iDataLength, String strDesc, Object strDefault)
    {
        this();
        this.init(record, strName, iDataLength, strDesc, strDefault);
    }
    /**
     * Creates a new object of the same class as this object.
     * @return     a clone of this instance.
     * @exception  CloneNotSupportedException  if the object's class does not support the <code>Cloneable</code> interface.
     * @see        java.lang.Cloneable
     */
    public Object clone() throws CloneNotSupportedException
    {
        BaseField field = new PasswordField(null, m_strFieldName, m_iMaxLength, m_strFieldDesc, null);
        field.setRecord(m_record);     // Set table without adding to table field list
        return field;
    }
    /**
     * Set up the default screen control for this field (A SPasswordField).
     * @param itsLocation Location of this component on screen (ie., GridBagConstraint).
     * @param targetScreen Where to place this component (ie., Parent screen or GridBagLayout).
     * @param converter The converter to set the screenfield to.
     * @param iDisplayFieldDesc Display the label? (optional).
     * @return Return the component or ScreenField that is created for this field.
     */
    public ScreenField setupDefaultView(ScreenLocation itsLocation, BasePanel targetScreen, Converter converter, int iDisplayFieldDesc)
    {
        return new SPasswordField(itsLocation, targetScreen, converter, iDisplayFieldDesc);
    }
    /**
     * Get the HTML Input Type.
     */
    public String getInputType(String strViewType)
    {
        if (TopScreen.HTML_TYPE.equalsIgnoreCase(strViewType))
            return "password";
        else //if (TopScreen.XML_TYPE.equalsIgnoreCase(strViewType))
            return "secret";
    }
}
